#' Function to preprocess Affymetrix Axiom arrays
#' 
#' This function preprocess the raw .CEL files from Affymetrix Axiom arrays
#' running the tools \code{apt-geno-qc} and \code{apt-probeset-genotype}
#' included into the Affymetrix Port Tools and generate the metrix, summary 
#' and genotype files need for \code{Ax2Mad} and \code{Ax2SnpMatrix}
#' 
#' @param cel.files Location where the raw .CEL files are placed.
#' @param output.path Location where the metrix, summary and genotype file will be stored.
#' @param analysis.path Path until Axiom library (containing DQC and Step 1 and Step 2 XML files).
#' @param xml.dqc XML definition of the QC metric that evaluates the overlap between the two homozygous peaks.
#' @param xml.config1 Configuration in XML for the first step of Axiom QC pipeline.
#' @param xml.config2 Configuration in XML for the first step of Aciom QC pipeline.
#' @param report.txt By default \code{'AxiomGT1.report.txt'}. Output name of the QC report generated by APT.
#' @param verbose By default FALSE. If TRUE the function will shows messages indicating the process.
#' @examples
#' \dontrun{
#' ax2apt( "celFiles", "outputFiles", "analysisFiles", 
#'    "analysisFiles/Axiom_GW_Hu_SNP.r6.apt-geno-qc.AxiomQC1.xml", 
#'    "Axiom_GW_Hu_SNP_LessThan96_Step1.r6.apt-probeset-genotype.AxiomGT1.xml", 
#'    "Axiom_GW_Hu_SNP_LessThan96_Step2.r6.apt-probeset-genotype.AxiomGT1.xml",
#'    temppath="tempPath", verbose=TRUE )
#' }
#' @export Ax2APT
Ax2APT <- function( cel.files, output.path, analysis.path, 
    xml.dqc, xml.config.1, xml.config.2, report.txt = "AxiomGT1.report.txt", 
    verbose = FALSE ) {

    # SETUPT
    OptionsAffy.Init("Ax2APT", verbose)
    # /SETUP

    # SYSTEM
    CheckSystem()
    # /SYSTEM

    if (Sys.info()[ "sysname" ] == "Linux") {
        apt.qc <- system.file("exec", .Platform$file.sep, "linux64.apt-geno-qc", package="affy2sv")
        apt.genotype <- system.file("exec", .Platform$file.sep, "linux64.apt-probeset-genotype", package="affy2sv")
    } else {
        if (Sys.info()["sysname"] == "Windows") {
            apt.qc <- system.file("exec", .Platform$file.sep, "win64.apt-geno-qc.exe", package="affy2sv")
            apt.genotype <- system.file("exec", .Platform$file.sep, "win64.apt-probeset-genotype.exe", package="affy2sv")
        } else {
            apt.qc <- system.file("exec", .Platform$file.sep, "mac64.apt-geno-qc", package="affy2sv")
            apt.genotype <- system.file("exec", .Platform$file.sep, "mac64.apt-probeset-genotype", package="affy2sv")
        }
    }


    # check directory exists
    xml.dqc <- file.path(analysis.path, xml.dqc, fsep=.Platform$file.sep)
    tempfile <- RandomName(prefix="tmp.ax.apt.")
    temppath <- RandomName(prefix="tmp.ax.apt.")


    unlink(temppath, recursive=TRUE, force=TRUE)
    dir.create(temppath, showWarnings=FALSE)


    # STEP 1: Analyses DQC
    ScreenAffy( "dqc analysis ", apt.qc, " ", apt.genotype );
    celds <- data.frame( c( "cel_files", list.celfiles( cel.files, full.names=TRUE ) ) );
    celnm <- paste0( tempfile, "1" )
    write.table( celds, file=paste0( celnm, ".1" ), quote=FALSE, row.names=FALSE, col.names=FALSE );

    qcfile <- apt_quality_control( apt.qc, paste0( celnm, ".1" ), analysis.path, xml.dqc, temppath, verbose );
    
    # STEP 2: Filter DQC
    ScreenAffy( "dqc filtering" );
    qcfile <- read.table( qcfile, sep="\t", header=TRUE, row.names=1 );
    selected <- rownames( qcfile[ qcfile[ , "axiom_dishqc_DQC" ] > 0.82, ] );

    celds <- as.character( celds[ ,1 ] );

    celds <- lapply( selected,
        function( item, all ) {
            for( ii in 1:length( all ) ) {
                if( length( grep( item, all[ ii ] ) ) != 0 ) {
                    return( all[ ii ] );
                }
            }
            return( "" );
        }, all=celds );
    celds <- data.frame( c( "cel_files", unlist( celds ) ) );
    write.table( celds, file=paste0( celnm, ".2" ), quote=FALSE, row.names=FALSE, col.names=FALSE );


    # STEP 3: Analyses QC
    ScreenAffy( "qc analysis" );
    apt_genotype_ax( apt.genotype, paste0( celnm, ".2" ), analysis.path, xml.config.1, temppath, verbose );
    qcfile <- read.table( file.path( temppath, report.txt, fsep=.Platform$file.sep ), header=TRUE, row.names=1, sep="\t" );
    selected <- rownames( qcfile[ qcfile[ , "call_rate" ] >= 97.0, ] );

    celds <- as.character( celds[ ,1 ] );

    celds <- lapply( selected,
        function( item, all ) {
            for( ii in 1:length( all ) ) {
                if( length( grep( item, all[ ii ] ) ) != 0 ) {
                    return( all[ ii ] );
                }
            }
            return( "" );
        }, all=celds );
    celds <- data.frame( c( "cel_files", unlist( celds ) ) );
    write.table( celds, file=paste0( celnm, ".3" ), quote=FALSE, row.names=FALSE, col.names=FALSE );

    # STEP 4: Filter CALL RATE
    ScreenAffy( "filtering call rate" );
    rpt <- read.delim( file.path( temppath, report.txt, fsep=.Platform$file.sep ), comment.char="#" );
    selected <- as.character( rpt[ rpt[ , "call_rate" ] > 97.0, "cel_files" ] );

    celds <- as.character( celds[ ,1 ] );

    celds <- lapply( selected,
        function( item, all ) {
            for( ii in 1:length( all ) ) {
                if( length( grep( item, all[ ii ] ) ) != 0 ) {
                    return( all[ ii ] );
                }
            }
            return( "" );
        }, all=celds );
    celds <- data.frame( c( "cel_files", unlist( celds ) ) );
    write.table( celds, file=paste0( celnm, ".4" ), quote=FALSE, row.names=FALSE, col.names=FALSE );

    # STEP 5: Genotype
    ScreenAffy( "genotyping" )
    apt_genotype_ax( apt.genotype, paste0( celnm, ".4" ), analysis.path, xml.config.2, temppath, verbose );


    # STEP 6: Generate the metrix file.
    metrics.fun(
        file.path( temppath, "AxiomGT1.snp-posteriors.txt", fsep=.Platform$file.sep ),
        file.path( temppath, "AxiomGT1.calls.txt", fsep=.Platform$file.sep ),
        file.path( temppath, "AxiomGT1.metrix.txt", fsep=.Platform$file.sep ),
        0 )

    # Moving
    unlink( output.path, recursive=TRUE, force=TRUE );
    dir.create( output.path, showWarnings=FALSE );
    file.rename( file.path( temppath, "AxiomGT1.calls.txt", fsep=.Platform$file.sep ) , file.path( output.path, "AxiomGT1.calls.txt", fsep=.Platform$file.sep ) );
    file.rename( file.path( temppath, "AxiomGT1.summary.txt", fsep=.Platform$file.sep ) , file.path( output.path, "AxiomGT1.summary.txt", fsep=.Platform$file.sep) );
    file.rename( file.path( temppath, "AxiomGT1.metrix.txt", fsep=.Platform$file.sep ) , file.path( output.path, "AxiomGT1.metrix.txt", fsep=.Platform$file.sep ) );

    #unlink( "temp", recursive=TRUE, force=TRUE );
    unlink( temppath, recursive=TRUE, force=TRUE );
}

